# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets
from shapely.geometry import Point
from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as Canvas
from matplotlib.patches import Polygon
import matplotlib as plt
import numpy as np
from numpy.linalg import norm
from copy import copy
from abc import abstractmethod

from sklearn.preprocessing import MinMaxScaler

from modules.tools_views import ViewManager, PointsOfInterest
from modules.plot_manager import POIPoint, ManagePlot


# Matplotlib canvas class to create figure
class MplCanvas(Canvas):
    def __init__(self):
        # Defines figure and one axis
        self.fig = Figure()
        self.ax = self.fig.add_subplot(111)

        # Defines the figure as a canvas
        Canvas.__init__(self, self.fig)
        Canvas.setSizePolicy(self, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        Canvas.updateGeometry(self)


class AbstractMplWidget(QtWidgets.QWidget):
    def __init__(self, plot_manager, parent=None):
        QtWidgets.QWidget.__init__(self, parent)  # Inherit from QWidget

        self.canvas = MplCanvas()  # Create canvas object
        self.canvas.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.canvas.setFocus()

        self.vbl = QtWidgets.QVBoxLayout()  # Set box for plotting
        self.vbl.addWidget(self.canvas)
        self.setLayout(self.vbl)
        self.setObjectName("widget")
        self.plot_manager: ManagePlot = plot_manager

        self.select_max_dist = 0.015
        self.snapping_margin = 0.03
        self.point_min_dist = 0.03
        self.point_creation_max_dist = 0.005

        # Sets up matplotlib event listeners
        self.canvas.mpl_connect("button_press_event", self.on_press)
        self.canvas.mpl_connect("motion_notify_event", self.on_move)
        self.canvas.mpl_connect("button_release_event", self.on_release)
        self.canvas.mpl_connect("key_press_event", self.on_key_press)

    def is_point_inside_room(self, point: Point):
        poly = Polygon(self.plot_manager.data_points.accessible_polygon)
        return poly.contains(point)[0]

    @abstractmethod
    def on_press(self, event):
        pass

    @abstractmethod
    def on_move(self, event):
        pass

    @abstractmethod
    def on_release(self, event):
        pass

    @abstractmethod
    def on_key_press(self, event):
        pass

    def remove_points_outside_poly(self):
        i = 0
        while i < len(self.plot_manager.data_points.points_of_interest) - 1:
            point = Point(self.plot_manager.data_points.points_of_interest[i].poi_position)
            is_inside_poly = self.is_point_inside_room(point)
            if not is_inside_poly:
                self.plot_manager.remove_poi_point(i)
            else:
                i += 1

    def draw(self):
        self.canvas.ax.clear()

        vertices = self.plot_manager.data_points.vertices
        if len(vertices):
            np_vertices = np.array(vertices)
            self.canvas.ax.scatter(np_vertices[:, 0], np_vertices[:, 1], s=1, color='lightgrey')

        poly_x, poly_y = self.plot_manager.return_poly_plot_points()
        if len(poly_x) != 0 and len(poly_y) != 0:
            poly_x.append(poly_x[0])
            poly_y.append(poly_y[0])
            self.canvas.ax.plot(poly_x, poly_y, color='black')

        target_x, target_y = self.plot_manager.return_target_plot_points()
        if len(target_x) != 0 and len(target_y) != 0:
            self.canvas.ax.plot(target_x, target_y, 'mo')

        station_point = self.plot_manager.get_base_station_point()
        if station_point is not None:
            self.canvas.ax.plot(station_point[0], station_point[1], 'ro')

        drone_point = self.plot_manager.get_drone_point()
        if drone_point is not None:
            self.canvas.ax.plot(drone_point[0], drone_point[1], 'go')

        self.canvas.ax.axis('off')
        self.canvas.ax.set_ylim(0, 1)
        self.canvas.ax.set_xlim(0, 1)
        self.canvas.fig.set_tight_layout(True)
        self.canvas.draw()


class DefaultMplWidget(AbstractMplWidget):
    def on_press(self, _):
        pass

    def on_move(self, _):
        pass

    def on_release(self, _):
        pass

    def on_key_press(self, _):
        pass


class EditMapMplWidget(AbstractMplWidget):
    def __init__(self, plot_manager, parent=None):
        super(EditMapMplWidget, self).__init__(plot_manager, parent)
        self.plot_manager.unselect_poly_point_idx()

    def on_press(self, event):
        if event.xdata is not None and event.ydata is not None:
            point_list = self.plot_manager.data_points.accessible_polygon
            click_point = [event.xdata, event.ydata]
            self.plot_manager.unselect_poly_point_idx()

            if event.dblclick:
                if len(point_list) < 2:
                    self.plot_manager.data_points.accessible_polygon.append(click_point)
                    self.plot_manager.selected_poly_point_idx = len(
                        self.plot_manager.data_points.accessible_polygon) - 1
                else:
                    point_list = copy(point_list)
                    point_list.append(point_list[0])

                    min_idx = None
                    min_prod = None
                    for idx in range(len(point_list) - 1):
                        a = np.array([point_list[idx][0], point_list[idx][1]])
                        b = np.array([point_list[idx + 1][0], point_list[idx + 1][1]])
                        c = np.array(click_point)

                        d = norm(np.cross(b - a, a - c) / norm(b - a))

                        if min_prod is None or d < min_prod:
                            min_prod = d
                            min_idx = idx

                    point_idx = None
                    if min_prod < self.point_creation_max_dist:
                        point_idx = min_idx

                    if point_idx is not None:
                        self.plot_manager.add_poly_point_to_idx(point_idx + 1, click_point)
                        self.plot_manager.selected_poly_point_idx = point_idx + 1
            elif len(point_list) > 1:
                point_dists = [Point(click_point).distance(Point(point)) for point in point_list]
                dist = min(point_dists)

                if dist <= self.select_max_dist:
                    self.plot_manager.selected_poly_point_idx = point_dists.index(dist)
                    self.plot_manager.holding_mouse = True

            self.draw()

    def on_move(self, event):
        if self.plot_manager.holding_mouse and self.plot_manager.is_poly_point_selected() and event.xdata is not None and event.ydata is not None:
            event_x = event.xdata
            event_y = event.ydata

            # Checks every point x and y values if they are within the margins of the selected point
            # If it's within the margin set the x and/or y to that value
            x_val, y_val = self.plot_manager.return_poly_plot_points()
            for idx, x in enumerate(x_val):
                # Doesn't check the values of the selected point
                if idx != self.plot_manager.selected_poly_point_idx and np.abs(x - event_x) < self.snapping_margin:
                    event_x = x
                    break

            for idx, y in enumerate(y_val):
                # Doesn't check the values of the selected point
                if idx != self.plot_manager.selected_poly_point_idx and np.abs(y - event_y) < self.snapping_margin:
                    event_y = y
                    break

            # Sets new x and y values for the selected point
            idx = self.plot_manager.selected_poly_point_idx
            self.plot_manager.set_poly_point(idx, [event_x, event_y])

            self.draw()

    def on_release(self, _):
        if self.plot_manager.is_poly_point_selected():
            self.plot_manager.holding_mouse = False
            self.remove_points_outside_poly()
            self.draw()

    def on_key_press(self, event):
        if self.plot_manager.is_poly_point_selected() and event.key == 'delete':
            self.plot_manager.remove_poly_point(self.plot_manager.selected_poly_point_idx)
            self.plot_manager.unselect_poly_point_idx()
            self.plot_manager.holding_mouse = False
            self.draw()

    def draw(self):
        super().draw()

        poly_x, poly_y = self.plot_manager.return_poly_plot_points()
        if len(poly_x) != 0 and len(poly_y) != 0:
            selected_idx = self.plot_manager.selected_poly_point_idx
            if selected_idx is not None:
                selected_x = poly_x[selected_idx]
                selected_y = poly_y[selected_idx]
                poly_x = poly_x[:selected_idx] + poly_x[selected_idx + 1:]
                poly_y = poly_y[:selected_idx] + poly_y[selected_idx + 1:]
                self.canvas.ax.plot(selected_x, selected_y, 'ro', markersize=8, markeredgewidth=2, markeredgecolor='b')
            self.canvas.ax.plot(poly_x, poly_y, 'ro')

            self.canvas.draw()


class EditPointsOfInterestMplWidget(AbstractMplWidget):
    def __init__(self, get_radio_btn_state, plot_manager, parent=None):
        super(EditPointsOfInterestMplWidget, self).__init__(plot_manager, parent)
        self.get_radio_btn_state = get_radio_btn_state
        self.plot_manager.unselect_poi_point_idx()

    def on_press(self, event):
        btn_state = self.get_radio_btn_state()
        if btn_state is not None and event.xdata is not None and event.ydata is not None:
            click_point = Point(event.xdata, event.ydata)

            # If click is not inside poly then it's ignored
            if not self.is_point_inside_room(click_point):
                return

            self.plot_manager.unselect_poi_point_idx()

            # With double click point is added
            point_xy = [event.xdata, event.ydata]
            if event.dblclick:
                point = POIPoint(poi_type=btn_state, poi_position=point_xy)

                # If correct state and drone exists or base station exists then add point
                if btn_state == PointsOfInterest.drone and not self.plot_manager.drone_point_exists() or \
                        btn_state == PointsOfInterest.base_station and not self.plot_manager.base_station_point_exists() or \
                        btn_state == PointsOfInterest.target:
                    self.plot_manager.data_points.points_of_interest.append(point)
                    self.plot_manager.selected_poi_point_idx = self.plot_manager.get_poi_point_idx(point_xy)

            else:
                # If distance between click and point is small enough then that point is selected
                if btn_state == PointsOfInterest.drone and self.plot_manager.drone_point_exists():
                    point_xy = self.plot_manager.get_drone_point()
                    dist = click_point.distance(Point(point_xy))
                    if dist <= self.select_max_dist:
                        self.plot_manager.holding_mouse = True

                elif btn_state == PointsOfInterest.base_station and self.plot_manager.base_station_point_exists():
                    point_xy = self.plot_manager.get_base_station_point()
                    dist = click_point.distance(Point(point_xy))
                    if dist <= self.select_max_dist:
                        self.plot_manager.holding_mouse = True

                elif btn_state == PointsOfInterest.target:
                    point_list = self.plot_manager.get_target_point_list()

                    if len(point_list) != 0:
                        # Distance between every point is calculated and min dist is selected
                        point_dists = [click_point.distance(Point(point)) for point in point_list]
                        dist = min(point_dists)
                        point_xy = point_list[point_dists.index(dist)]

                        if dist <= self.select_max_dist:
                            self.plot_manager.holding_mouse = True

                if self.plot_manager.holding_mouse:
                    self.plot_manager.selected_poi_point_idx = self.plot_manager.get_poi_point_idx(point_xy)

        self.draw()

    def on_move(self, event):
        btn_state = self.get_radio_btn_state()
        if btn_state is not None and self.plot_manager.holding_mouse and event.xdata is not None and event.ydata is not None:
            # If click is not inside poly then it's ignored
            if not self.is_point_inside_room(Point(event.xdata, event.ydata)):
                self.plot_manager.holding_mouse = False
                return

            self.plot_manager.data_points.points_of_interest[self.plot_manager.selected_poi_point_idx].poi_position = [
                event.xdata, event.ydata]

            self.draw()

    def on_release(self, event):
        self.plot_manager.holding_mouse = False
        self.draw()

    def on_key_press(self, event):
        btn_state = self.get_radio_btn_state()
        if btn_state is not None and event.key == 'delete':
            if self.plot_manager.is_poi_point_selected():
                self.plot_manager.remove_poi_point(self.plot_manager.selected_poi_point_idx)
                self.plot_manager.unselect_poi_point_idx()
                
            self.draw()

    def draw(self):
        super().draw()

        if self.plot_manager.is_poi_point_selected():
            point = self.plot_manager.data_points.points_of_interest[
                self.plot_manager.selected_poi_point_idx]

            point_color = 'yo'
            if point.poi_type == PointsOfInterest.target:
                point_color = 'mo'
            elif point.poi_type == PointsOfInterest.drone:
                point_color = 'go'
            elif point.poi_type == PointsOfInterest.base_station:
                point_color = 'ro'

            self.canvas.ax.plot(point.poi_position[0], point.poi_position[1], point_color, markersize=8, markeredgewidth=2, markeredgecolor='b')

            self.canvas.draw()


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(874, 587)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout.setSpacing(10)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setMaximumSize(QtCore.QSize(16777215, 30))
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.pushButton = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton.setCheckable(True)
        self.pushButton.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.pushButton.setObjectName("pushButton")
        self.verticalLayout.addWidget(self.pushButton)
        self.pushButton_2 = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_2.setCheckable(True)
        self.pushButton_2.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.pushButton_2.setObjectName("pushButton_2")
        self.verticalLayout.addWidget(self.pushButton_2)
        self.pushButton_3 = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_3.setCheckable(True)
        self.pushButton_3.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.pushButton_3.setObjectName("pushButton_3")
        self.verticalLayout.addWidget(self.pushButton_3)
        self.gridLayout.addLayout(self.verticalLayout, 0, 3, 1, 1)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_2.setSpacing(10)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setMaximumSize(QtCore.QSize(16777215, 30))
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)

        self.btn_group = QtWidgets.QButtonGroup(self.centralwidget)

        self.radioButton = QtWidgets.QRadioButton(self.centralwidget)
        self.radioButton.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.radioButton.setObjectName("radioButton")
        self.radioButton.setChecked(True)
        self.btn_group.addButton(self.radioButton)
        self.verticalLayout_2.addWidget(self.radioButton)

        self.radioButton_2 = QtWidgets.QRadioButton(self.centralwidget)
        self.radioButton_2.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.radioButton_2.setObjectName("radioButton_2")
        self.btn_group.addButton(self.radioButton_2)
        self.verticalLayout_2.addWidget(self.radioButton_2)

        self.radioButton_3 = QtWidgets.QRadioButton(self.centralwidget)
        self.radioButton_3.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.radioButton_3.setObjectName("radioButton_3")
        self.btn_group.addButton(self.radioButton_3)
        self.verticalLayout_2.addWidget(self.radioButton_3)

        self.gridLayout.addLayout(self.verticalLayout_2, 1, 3, 1, 1)

        self.view_manager = ViewManager(self.centralwidget)

        self.layout_3 = self.view_manager.get_properties()
        self.gridLayout.addLayout(self.layout_3, 2, 3, 1, 1)

        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_5.setSpacing(10)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_8 = QtWidgets.QLabel(self.centralwidget)
        self.label_8.setEnabled(True)
        self.label_8.setMaximumSize(QtCore.QSize(16777215, 30))
        self.label_8.setObjectName("label_8")
        self.verticalLayout_5.addWidget(self.label_8)

        self.plot_manager = ManagePlot()
        self.widget = DefaultMplWidget(self.plot_manager, self.centralwidget)
        self.widget.draw()
        self.verticalLayout_5.addWidget(self.widget)

        self.gridLayout.addLayout(self.verticalLayout_5, 0, 0, 3, 2)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 874, 22))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))

        self.label.setText(_translate("MainWindow", "File"))
        self.pushButton.setText(_translate("MainWindow", "Open PLY"))
        self.pushButton_2.setText(_translate("MainWindow", "Open JSON"))
        self.pushButton_3.setText(_translate("MainWindow", "Save JSON"))

        self.label_2.setText(_translate("MainWindow", "Tools"))
        self.radioButton.setText(_translate("MainWindow", "Viewer"))
        self.radioButton_2.setText(_translate("MainWindow", "Edit map"))
        self.radioButton_3.setText(_translate("MainWindow", "Edit points of interest"))

        self.label_8.setText(_translate("MainWindow", "Map"))
