import os
import sys
import matplotlib
matplotlib.use('QT5Agg')

path_current_dir = os.path.dirname(__file__)
if path_current_dir not in sys.path:
    sys.path.append(path_current_dir)

from PyQt5.QtWidgets import QMainWindow
from PyQt5 import QtWidgets, QtCore


from MainWindow import Ui_MainWindow, DefaultMplWidget, EditMapMplWidget, EditPointsOfInterestMplWidget
from modules.tools_views import DefaultView, EditMapView, EditPointsOfInterestView
from modules.file_manager import FileLoader


def clear_layout(layout):
    for i in reversed(range(layout.count())):
        if not isinstance(layout.itemAt(i), QtWidgets.QSpacerItem):
            layout.itemAt(i).widget().deleteLater()


class MainWindow(QMainWindow, Ui_MainWindow):
    def __init__(self, *args, obj=None, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        self.setupUi(self)

        self.radioButton.toggled.connect(self.default_view)
        self.radioButton_2.toggled.connect(self.edit_map_view)
        self.radioButton_3.toggled.connect(self.edit_points_of_interest_view)

        self.file_loader = FileLoader(self)

        self.pushButton.toggled.connect(self.load_ply)
        self.pushButton_2.toggled.connect(self.load_json)
        self.pushButton_3.toggled.connect(self.save_json)

    def clear_and_set_view(self):
        clear_layout(self.layout_3)
        self.layout_3 = self.view_manager.get_properties()
        self.gridLayout.addLayout(self.layout_3, 2, 3, 1, 1)

    def load_ply(self):
        if self.pushButton.isChecked():
            data_points = self.file_loader.load_ply()
            if data_points is not None:
                self.plot_manager.data_points = data_points
                self.widget.remove_points_outside_poly()
            self.widget.draw()
            self.pushButton.setChecked(False)

    def load_json(self):
        if self.pushButton_2.isChecked():
            data_points = self.file_loader.load_json_from_file()
            if data_points is not None:
                self.plot_manager.data_points = data_points
                self.widget.remove_points_outside_poly()
            self.widget.draw()
            self.pushButton_2.setChecked(False)

    def save_json(self):
        if self.pushButton_3.isChecked():
            self.file_loader.save_json(self.plot_manager.data_points)
            self.pushButton_3.setChecked(False)

    def default_view(self):
        if self.radioButton.isChecked():
            self.view_manager.set_view(DefaultView())

            clear_layout(self.verticalLayout_5)

            self.verticalLayout_5 = QtWidgets.QVBoxLayout()
            self.verticalLayout_5.setContentsMargins(10, 10, 10, 10)
            self.verticalLayout_5.setSpacing(10)
            self.verticalLayout_5.setObjectName("verticalLayout_5")
            self.label_8 = QtWidgets.QLabel(self.centralwidget)
            self.label_8.setEnabled(True)
            self.label_8.setMaximumSize(QtCore.QSize(16777215, 30))
            self.label_8.setObjectName("label_8")

            _translate = QtCore.QCoreApplication.translate
            self.label_8.setText(_translate("MainWindow", "Map"))

            self.verticalLayout_5.addWidget(self.label_8)

            self.widget = DefaultMplWidget(self.plot_manager, self.centralwidget)
            self.widget.draw()
            self.verticalLayout_5.addWidget(self.widget)
            self.gridLayout.addLayout(self.verticalLayout_5, 0, 0, 3, 2)

            self.clear_and_set_view()

    def edit_map_view(self):
        if self.radioButton_2.isChecked():
            self.view_manager.set_view(EditMapView())

            clear_layout(self.verticalLayout_5)

            self.verticalLayout_5 = QtWidgets.QVBoxLayout()
            self.verticalLayout_5.setContentsMargins(10, 10, 10, 10)
            self.verticalLayout_5.setSpacing(10)
            self.verticalLayout_5.setObjectName("verticalLayout_5")
            self.label_8 = QtWidgets.QLabel(self.centralwidget)
            self.label_8.setEnabled(True)
            self.label_8.setMaximumSize(QtCore.QSize(16777215, 30))
            self.label_8.setObjectName("label_8")

            _translate = QtCore.QCoreApplication.translate
            self.label_8.setText(_translate("MainWindow", "Map"))

            self.verticalLayout_5.addWidget(self.label_8)

            self.widget = EditMapMplWidget(self.plot_manager, self.centralwidget)
            self.widget.draw()
            self.verticalLayout_5.addWidget(self.widget)
            self.gridLayout.addLayout(self.verticalLayout_5, 0, 0, 3, 2)

            self.clear_and_set_view()

    def edit_points_of_interest_view(self):
        if self.radioButton_3.isChecked():
            self.view_manager.set_view(EditPointsOfInterestView())

            clear_layout(self.verticalLayout_5)

            self.verticalLayout_5 = QtWidgets.QVBoxLayout()
            self.verticalLayout_5.setContentsMargins(10, 10, 10, 10)
            self.verticalLayout_5.setSpacing(10)
            self.verticalLayout_5.setObjectName("verticalLayout_5")
            self.label_8 = QtWidgets.QLabel(self.centralwidget)
            self.label_8.setEnabled(True)
            self.label_8.setMaximumSize(QtCore.QSize(16777215, 30))
            self.label_8.setObjectName("label_8")

            _translate = QtCore.QCoreApplication.translate
            self.label_8.setText(_translate("MainWindow", "Map"))

            self.verticalLayout_5.addWidget(self.label_8)

            self.widget = EditPointsOfInterestMplWidget(self.view_manager.get_radio_btn_state, self.plot_manager, self.centralwidget)
            self.widget.draw()
            self.verticalLayout_5.addWidget(self.widget)
            self.gridLayout.addLayout(self.verticalLayout_5, 0, 0, 3, 2)

            self.clear_and_set_view()


if __name__ == '__main__':

    app = QtWidgets.QApplication(sys.argv)
    window = MainWindow()
    window.show()
    app.exec()
