from abc import ABC, abstractmethod
from PyQt5 import QtCore, QtGui, QtWidgets


class QtPropertyDecorator(ABC):
    def __init__(self, qt_obj, obj_name: str):
        self.qt_obj = qt_obj
        self.obj_name = obj_name

    @abstractmethod
    def get_element(self):
        pass


class QLongLabelDecorator(QtPropertyDecorator):
    def get_element(self):
        self.qt_obj.setWordWrap(True)
        self.qt_obj.setObjectName(self.obj_name)
        self.qt_obj.setMaximumSize(QtCore.QSize(210, 16777215))
        return self.qt_obj


class QShortLabelDecorator(QtPropertyDecorator):
    def get_element(self):
        self.qt_obj.setWordWrap(True)
        self.qt_obj.setObjectName(self.obj_name)
        self.qt_obj.setMaximumSize(QtCore.QSize(210, 30))
        return self.qt_obj


class QLayoutDecorator(QtPropertyDecorator):
    def get_element(self):
        self.qt_obj.setContentsMargins(10, 10, 10, 10)
        self.qt_obj.setSpacing(10)
        self.qt_obj.setObjectName(self.obj_name)
        return self.qt_obj


class QRadioBtnDecorator(QtPropertyDecorator):
    def get_element(self):
        self.qt_obj.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.qt_obj.setObjectName(self.obj_name)
        return self.qt_obj
