import json
from copy import deepcopy

import numpy as np
from PyQt5.QtWidgets import QFileDialog
from plyfile import PlyData, PlyElement

from modules.plot_manager import DataPoints, POIPoint
from modules.tools_views import PointsOfInterest

from backend_main import process_ply


class FileLoader:
    def __init__(self, app):
        self.app_pointer = app
        # self.data_max: float = float('-Inf')
        # self.data_min: float = float('Inf')

        self.plot_y = 480
        self.plot_x = 640

    # def reset_min_max(self):
    #     self.data_max = float('-Inf')
    #     self.data_min = float('Inf')
    #
    # def get_min_max(self, data):
    #     np_data = np.array(data)
    #     np_max = np.max(np_data)
    #     np_min = np.min(np_data)
    #     self.data_max = np_max if np_max > self.data_max else self.data_max
    #     self.data_min = np_min if np_min < self.data_min else self.data_min
    #
    # def normalize(self, data, optional_max=float('-Inf')):
    #     np_data = np.array(data)
    #     data_max = optional_max if optional_max > self.data_max else self.data_max
    #     return ((np_data - self.data_min) / (data_max - self.data_min)).tolist()
    #
    # def un_normalize(self, data, optional_max=float('-Inf')):
    #     np_data = np.array(data)
    #     data_max = optional_max if optional_max > self.data_max else self.data_max
    #     return (np_data * (data_max - self.data_min) + self.data_min).tolist()
    #
    # def normalize_points(self, data: DataPoints):
    #     if len(data.accessible_polygon) > 0:
    #         data.accessible_polygon = self.normalize(data.accessible_polygon)
    #
    #     if len(data.points_of_interest) > 0:
    #         for idx in range(len(data.points_of_interest)):
    #             data.points_of_interest[idx].poi_position = self.normalize(data.points_of_interest[idx].poi_position)
    #
    #     return data
    #
    # def un_normalize_points(self, data: DataPoints):
    #     if len(data.accessible_polygon) > 0:
    #         data.accessible_polygon = self.un_normalize(data.accessible_polygon)
    #
    #     if len(data.points_of_interest) > 0:
    #         for idx in range(len(data.points_of_interest)):
    #             data.points_of_interest[idx].poi_position = self.un_normalize(data.points_of_interest[idx].poi_position)
    #
    #     return data

    def normalize(self, data, data_max):
        np_data = np.array(data)
        data_min = 0
        return ((np_data - data_min) / (data_max - data_min)).tolist()

    def un_normalize(self, data, data_max):
        np_data = np.array(data)
        data_min = 0
        return (np_data * (data_max - data_min) + data_min).tolist()

    def load_ply(self):
        result = None
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        file, _ = QFileDialog.getOpenFileName(self.app_pointer, 'Open file', '', '(*.ply)', options=options)

        if file and file.endswith('.ply'):
            data_dict = process_ply(file)
            result = self.load_json(data_dict)
        return result

    def load_json_from_file(self):
        result = None
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        file, _ = QFileDialog.getOpenFileName(self.app_pointer, 'Open file', '', '(*.json)', options=options)

        if file and file.endswith('.json'):
            with open(file) as f:
                data_dict = json.load(f)
                result = self.load_json(data_dict)
        return result

    def load_json(self, data):
        plot_points = DataPoints()
        # self.reset_min_max()
        if 'accessible_polygon' in data.keys() and len(data['accessible_polygon']) > 0:
            plot_points.accessible_polygon = data['accessible_polygon']
            x = [point[0] for point in plot_points.accessible_polygon]
            y = [point[1] for point in plot_points.accessible_polygon]

            x = self.normalize(x, self.plot_x)
            y = self.normalize(y, self.plot_y)

            point_list = []
            for idx in range(len(x)):
                point_list.append([x[idx], y[idx]])

            plot_points.accessible_polygon = point_list

            # self.get_min_max(plot_points.accessible_polygon)

        if 'points_of_interest' in data.keys() and len(data['points_of_interest']) > 0:
            poi_list = []
            for item in data['points_of_interest']:
                if item['poi_type'] in [e for e in dir(PointsOfInterest) if not e.startswith('_')]:
                    x = self.normalize(item['poi_position'][0], self.plot_x)
                    y = self.normalize(item['poi_position'][1], self.plot_y)
                    poi = POIPoint(
                        poi_id=item['poi_id'],
                        poi_type=PointsOfInterest[item['poi_type']],
                        poi_position=[x, y]
                    )
                    poi_list.append(poi)
                    # self.get_min_max(item['poi_position'])

            plot_points.points_of_interest = poi_list

        if 'room_id' in data.keys() and data['room_id'] is not None:
            plot_points.room_id = data['room_id']

        if 'vertices' in data.keys() and data['vertices'] is not None:
            plot_points.vertices = data['vertices']

        # plot_points = self.normalize_points(plot_points)
        return plot_points

    def save_json(self, data):
        data = deepcopy(data)

        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        file_name, _ = QFileDialog.getSaveFileName(self.app_pointer, 'Save file', '', '(*.json)', options=options)

        if file_name:
            if not file_name.endswith('.json'):
                file_name = f'{file_name}.json'

            with open(file_name, 'w') as f:
                if len(data.accessible_polygon) > 0:
                    x = [point[0] for point in data.accessible_polygon]
                    y = [point[1] for point in data.accessible_polygon]

                    x = self.un_normalize(x, self.plot_x)
                    y = self.un_normalize(y, self.plot_y)

                    point_list = []
                    for idx in range(len(x)):
                        point_list.append([x[idx], y[idx]])

                    data.accessible_polygon = point_list

                    # data.accessible_polygon = self.un_normalize(data.accessible_polygon)

                if len(data.points_of_interest) > 0:
                    for idx in range(len(data.points_of_interest)):
                        # data.points_of_interest[idx].poi_position = self.un_normalize(data.points_of_interest[idx].poi_position)
                        x = self.un_normalize(data.points_of_interest[idx].poi_position[0], self.plot_x)
                        y = self.un_normalize(data.points_of_interest[idx].poi_position[1], self.plot_y)
                        data.points_of_interest[idx].poi_position = [x, y]

                # un_norm_data = self.un_normalize_points(data).__dict__
                un_norm_data = data.__dict__

                for idx in range(len(un_norm_data['points_of_interest'])):
                    poi = un_norm_data['points_of_interest'][idx].__dict__
                    poi['poi_type'] = poi['poi_type'].name
                    poi['poi_id'] = idx
                    un_norm_data['points_of_interest'][idx] = poi

                json.dump(un_norm_data, f, indent=4)
