import numpy as np
from shapely.geometry.linestring import LineString


class Point():
    def __init__(self, x, y):
        self.x = x
        self.y = y

    @property
    def as_list(self):
        return [float(self.x), float(self.y)]

    def distance(self, point):
        return np.sqrt((self.x - point.x)**2 + (self.y - point.y)**2)


class Line():
    def __init__(self, p1, p2, angle=None):
        if isinstance(p1, Point):
            self.start_point = p1
        else:
            self.start_point = Point(p1[0], p1[1])
        if isinstance(p2, Point):
            self.end_point = p2
        else:
            self.end_point = Point(p2[0], p2[1])

        # order points so start closer to 0,0
        p1_dist = np.sqrt((self.start_point.x - 0)**2 + (self.start_point.y - 0)**2)
        p2_dist = np.sqrt((self.end_point.x - 0)**2 + (self.end_point.y - 0)**2)
        if p2_dist < p1_dist:
            tmp_p = self.start_point
            self.start_point = self.end_point
            self.end_point = tmp_p

        if angle is None:
            self.angle = np.arctan2(self.end_point.y - self.start_point.y, self.end_point.x - self.start_point.x)
        else:
            self.angle = angle

        self.line = LineString((self.start_point.as_list, self.end_point.as_list))

    def delta_angle(self, angle):
        return np.atan2(np.sin(angle - self.angle), np.cos(angle - self.angle))

    def intersects_line(self, line):
        return self.line.intersection(line.line) if self.line.intersects(line.line) else False

    @property
    def as_list(self):
        return [*self.start_point.as_list, *self.end_point.as_list]

    @property
    def midpoint(self):
        return Point(x=(self.start_point.x + self.end_point.x) / 2, y=(self.start_point.y + self.end_point.y) / 2)

    @property
    def length(self):
        return np.sqrt((self.start_point.x - self.end_point.x)**2 + (self.start_point.y - self.end_point.y)**2)

    @property
    def angle_as_sin_cos(self):
        return np.sin(self.angle), np.cos(self.angle)