from typing import List
from dataclasses import dataclass, field

from modules.tools_views import PointsOfInterest


@dataclass
class POIPoint:
    poi_type: PointsOfInterest
    poi_position: List[int]
    poi_id: int or None = None


@dataclass
class DataPoints:
    room_id: int = 0
    accessible_polygon: list = field(default_factory=list)
    vertices: list = field(default_factory=list)
    points_of_interest: List[POIPoint] = field(default_factory=list)


@dataclass
class ManagePlot:
    data_points: DataPoints = DataPoints()

    selected_poly_point_idx: int or None = None
    selected_poi_point_idx: int or None = None
    holding_mouse: bool = False

    def get_base_station_point(self):
        possible_point = [point.poi_position for point in self.data_points.points_of_interest if
                          point.poi_type == PointsOfInterest.base_station]

        return possible_point[0] if len(possible_point) > 0 else None

    def get_drone_point(self):
        possible_point = [point.poi_position for point in self.data_points.points_of_interest if
                          point.poi_type == PointsOfInterest.drone]

        return possible_point[0] if len(possible_point) > 0 else None

    def base_station_point_exists(self):
        return PointsOfInterest.base_station in [point.poi_type for point in self.data_points.points_of_interest]

    def drone_point_exists(self):
        return PointsOfInterest.drone in [point.poi_type for point in self.data_points.points_of_interest]

    def get_target_point_list(self):
        return [point.poi_position for point in self.data_points.points_of_interest if
                point.poi_type == PointsOfInterest.target]

    def remove_poi_point(self, idx: int):
        self.data_points.points_of_interest.remove(self.data_points.points_of_interest[idx])

    def set_poi_point(self, idx: int, point: POIPoint):
        self.data_points.points_of_interest[idx] = point

    def get_poi_point_idx(self, point: List[int]):
        return [point.poi_position for point in self.data_points.points_of_interest].index(point)

    def unselect_poi_point_idx(self):
        self.selected_poi_point_idx = None

    def is_poi_point_selected(self):
        return self.selected_poi_point_idx is not None

    def return_target_plot_points(self):
        return [point.poi_position[0] for point in self.data_points.points_of_interest if
                point.poi_type == PointsOfInterest.target], \
               [point.poi_position[1] for point in self.data_points.points_of_interest if
                point.poi_type == PointsOfInterest.target]

    def unselect_poly_point_idx(self):
        self.selected_poly_point_idx = None

    def is_poly_point_selected(self):
        return self.selected_poly_point_idx is not None

    def add_poly_point_to_idx(self, idx, point: List[int]):
        self.data_points.accessible_polygon = self.data_points.accessible_polygon[:idx] + [point] + \
                                              self.data_points.accessible_polygon[idx:]

    def set_poly_point(self, idx: int, point: List[int]):
        self.data_points.accessible_polygon[idx] = point

    def remove_poly_point(self, idx: int):
        self.data_points.accessible_polygon.remove(self.data_points.accessible_polygon[idx])

    def return_poly_plot_points(self):
        return [point[0] for point in self.data_points.accessible_polygon], \
               [point[1] for point in self.data_points.accessible_polygon]
