from plyfile import PlyData  # pip install plyfile
import numpy as np
import sys
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
import cv2


class PlyReader:

    @staticmethod
    def read(path):
        with open(path, 'rb') as f:
            plydata = PlyData.read(f)

        vertices_x = plydata.elements[0].data['x']
        vertices_y = plydata.elements[0].data['y']
        vertices_z = plydata.elements[0].data['z']

        return [vertices_x, vertices_y, vertices_z]

    @staticmethod
    def points_3D_to_image(vertices, show=False):
        if 'win32' in sys.platform:
            fig, ax = plt.subplots(1,1)
            canvas = fig.canvas
        else:
            fig = plt.figure()
            canvas = FigureCanvas(fig)
        ax = fig.gca()
        ax.axis('off')
        # plt.axis('scaled')

        ax.scatter(vertices[0], vertices[1], s=1)
        if show:
            plt.show(block=False)

        canvas.draw()

        width, height = fig.get_size_inches() * fig.get_dpi()
        image = np.frombuffer(canvas.tostring_rgb(), dtype='uint8').reshape(int(height), int(width), 3)
        if not show:
            plt.close(fig)
        return image

    @staticmethod
    def plot_with_gt(vertices, lines):
        if 'win32' in sys.platform:
            fig, ax = plt.subplots(1,1)
            canvas = fig.canvas
        else:
            fig = plt.figure()
            canvas = FigureCanvas(fig)
        ax = fig.gca()
        ax.axis('off')
        # plt.axis('scaled')

        ax.scatter(vertices[0], vertices[1], s=1)
        for line in lines:
            x1, y1, x2, y2 = line[0]
            xs = [x1 ,x2]
            ys = [y1, y2]
            ax.plot(xs, ys)
        plt.show(block=False)

        canvas.draw()

        width, height = fig.get_size_inches() * fig.get_dpi()
        image = np.frombuffer(canvas.tostring_rgb(), dtype='uint8').reshape(int(height), int(width), 3)
        return image

    @staticmethod
    def plot_3D(vertices, show=False):
        fig = plt.figure(figsize=(10, 10))
        ax = Axes3D(fig)

        ax.scatter(vertices[0], vertices[1], vertices[2])
        if show:
            plt.show()

    @staticmethod
    def purge_points(image):
        r=10
        threshold_pixels = 5

        map_binary = np.array(np.where(image == (255, 255, 255), 0, 1), dtype=bool)[:, :, 0]

        new_image = map_binary.copy()
        for i in range(map_binary.shape[0]):
            for j in range(map_binary.shape[1]):
                if bool(map_binary[i, j]):
                    if np.sum(map_binary[:i-r, j-10:j+10]) > threshold_pixels and np.sum(map_binary[i+r:, j-10:j+10]) > threshold_pixels \
                            and np.sum(map_binary[i-r:i+r, :j-10]) > threshold_pixels and np.sum(map_binary[i-r:i+r, j+10:]) > threshold_pixels:
                        new_image[i, j] = False

        new_image = np.array(np.where(np.tile(new_image[:,:,np.newaxis], (1,1,3)), image, (255,255,255)), dtype=np.uint8)
        return new_image


if __name__ == '__main__':
    path = r'/Users/aigaandrijanova/Documents/GitHub/alarm_pointcloud_backend_2021_Q4/data/input/master_DonMadden_P1_20_D_1_1_1.ply'

    vertices = PlyReader.read(path)
    image = PlyReader.points_3D_to_image(vertices, True)
    PlyReader.purge_points(image)
    # PlyReader.plot_3D(vertices, False)
